# **vamOS 项目文档**

---

## **目录**

1. [项目概述](#项目概述)  
2. [项目架构](#项目架构)  
3. [搭建思路与过程](#搭建思路与过程)  
4. [各模块设计方案](#各模块设计方案)  
   - [内核主程序 (`kernel.c`)](#内核主程序-kernelc)  
   - [内存管理模块 (`memory.c`)](#内存管理模块-memoryc)  
   - [进程管理模块 (`process.c`)](#进程管理模块-processc)  
   - [文件系统模块 (`filesystem.c`)](#文件系统模块-filesystemc)  
   - [字符串操作模块 (`string.c`)](#字符串操作模块-stringc)  
   - [互斥锁模块 (`mutex.c`)](#互斥锁模块-mutexc)  
   - [同步机制模块 (`sync.c`)](#同步机制模块-syncc)  
5. [创新点与对操作系统教学的帮助](#创新点与对操作系统教学的帮助)  
6. [编译与运行](#编译与运行)  
7. [测试](#测试)  
8. [未来改进方向](#未来改进方向)  

---

## **1. 项目概述**

**项目名称**：vamOS  
**目标**：开发一个简单的 RISC-V 内核，具备内存管理、任务调度、文件系统、并发控制等基本功能，为操作系统教学提供实践平台。  
**用途**：帮助学生深入理解操作系统的核心概念，包括内核设计、内存管理、进程调度、文件系统和并发控制。

---

## **2. 项目架构**

项目的核心模块包括：

1. **内核主程序**：处理命令行输入和系统初始化。  
2. **内存管理模块**：提供动态内存分配和释放功能。  
3. **进程管理模块**：实现简单的任务调度和上下文切换。  
4. **文件系统模块**：提供基于内存的文件系统，支持基本的文件操作。  
5. **字符串操作模块**：替代标准库的字符串函数，适用于裸机开发。  
6. **互斥锁模块**：实现并发控制，防止资源竞争。  
7. **同步机制模块**：提供简单的信号量机制，控制任务执行顺序。

---

## **3. 搭建思路与过程**

### **3.1 环境搭建**

1. **工具链**：使用 **RISC-V GNU 工具链** (`riscv64-unknown-elf-gcc`) 进行交叉编译。  
2. **模拟器**：使用 **QEMU** (`qemu-system-riscv64`) 运行内核。  
3. **Docker 支持**：通过 Dockerfile 搭建开发环境，简化依赖安装。

### **3.2 项目结构**

```
vamos/
│-- Dockerfile           # 开发环境的 Docker 配置文件
│-- linker.ld            # 链接脚本，定义内核内存布局
│-- start.S              # 内核入口的启动代码
│-- kernel.h             # 头文件
│-- kernel.c             # 内核主程序以及shell
│-- memory.c             # 内存管理模块
│-- process.c            # 进程管理模块
│-- filesystem.c         # 文件系统模块
│-- string.c             # 自定义字符串操作函数
│-- string.h             # 字符串操作函数的声明
│-- mutex.c              # 互斥锁模块
│-- sync.c               # 同步机制模块
│-- image                # 测试通过的截图
└-- README.md            # 项目简介与编译运行说明
```

---

## **4. 各模块设计方案**

### **4.1 内核主程序 (`kernel.c`)**

- **功能**：
  - 初始化系统、处理用户输入、执行命令。
- **关键函数**：
  - `main()`：系统入口，启动 Shell。  
  - `handle_command()`：解析和执行用户命令。  

### **4.2 内存管理模块 (`memory.c`)**

- **功能**：
  - 动态内存分配和释放。
- **关键函数**：
  - `simple_malloc(size)`：分配指定大小的内存。  
  - `simple_free()`：释放已分配的内存。  

### **4.3 进程管理模块 (`process.c`)**

- **功能**：
  - 简单的任务调度和上下文切换。
- **关键函数**：
  - `schedule()`：轮流调度任务。  
  - `task1()` / `task2()`：示例任务。  

### **4.4 文件系统模块 (`filesystem.c`)**

- **功能**：
  - 基于内存的简单文件系统，支持创建、删除、读写文件。
- **关键函数**：
  - `fs_init()`：初始化文件系统。  
  - `fs_create(name)`：创建文件。  
  - `fs_delete(name)`：删除文件。  
  - `fs_write(name, data, size)`：写入数据。  
  - `fs_read(name, buffer, size)`：读取数据。  
  - `fs_list()`：列出所有文件。  

### **4.5 字符串操作模块 (`string.c`)**

- **功能**：
  - 提供字符串操作函数，替代标准库。
- **关键函数**：
  - `strncpy_()`、`strchr_()`、`strcmp_()`、`strlen_()`、`strncmp_()`。  

### **4.6 互斥锁模块 (`mutex.c`)**

- **功能**：
  - 防止多个任务同时访问共享资源。
- **关键函数**：
  - `mutex_init()`：初始化互斥锁。  
  - `mutex_lock()`：获取互斥锁。  
  - `mutex_unlock()`：释放互斥锁。  

### **4.7 同步机制模块 (`sync.c`)**

- **功能**：
  - 通过信号量控制任务的执行顺序。
- **关键函数**：
  - `semaphore_init()`：初始化信号量。  
  - `semaphore_wait()`：等待信号量。  
  - `semaphore_signal()`：释放信号量。  

---

## **5. 创新点与对操作系统教学的帮助**

1. **模块化设计**：各个功能模块清晰独立，便于逐步学习操作系统核心概念。相比xv6有着更为简单的文件结构，更简单的扩展方式，无需阅读大量手册。
2. **基本内存管理**：通过动态内存分配和释放，理解内存管理的基础。  
3. **进程调度和并发控制**：通过任务调度、互斥锁和信号量，学习并发编程和同步机制。  
4. **文件系统基础**：通过内存文件系统，理解文件的创建、删除、读写等操作。  
5. **Shell 交互**：提供命令行操作，使用简便。

---

## **6. 编译与运行**

### **编译命令**

```bash
riscv64-unknown-elf-gcc -nostdlib -mcmodel=medany -T linker.ld -o kernel.elf start.S kernel.c string.c
```

### **运行命令**

```bash
qemu-system-riscv64 -machine virt -kernel kernel.elf -nographic
```

---

## **7. 测试**

1. **内存管理测试**  
![memory](image/memory.png)
2. **进程调度测试**  
![process](image/process.png)
3. **文件系统测试**  
![file_system](image/file.png)
4. **并发相关测试**
![sync](image/sync.png)  
6. **help测试**  
![help](image/help.png)  

---

## **8. 未来改进方向**

1. **支持更多调度算法**：优先级调度,RR调度等调度策略。  
2. **增强文件系统**：支持目录结构,持久化存储以及链接等操作。  
3. **中断处理**：引入硬件中断和软件中断。  
4. **内存管理**：实现更复杂的内存管理机制，如段式或者页式管理。 
 
