// filesystem.c

#include "filesystem.h"
#include "kernel.h"
#include "string.h"


// 文件系统存储
static File files[MAX_FILES];

// 初始化文件系统
void fs_init() {
    for (int i = 0; i < MAX_FILES; i++) {
        files[i].in_use = 0;
    }
    uart_puts("File system initialized.\n");
}

// 创建文件
int fs_create(const char *filename) {
    for (int i = 0; i < MAX_FILES; i++) {
        if (!files[i].in_use) {
            strncpy_(files[i].name, filename, MAX_FILENAME_LEN - 1);
            files[i].name[MAX_FILENAME_LEN - 1] = '\0';
            files[i].size = 0;
            files[i].in_use = 1;
            uart_puts("File created: ");
            uart_puts(filename);
            uart_putc('\n');
            return 0;
        }
    }
    uart_puts("Error: File system is full.\n");
    return -1;
}

// 删除文件
int fs_delete(const char *filename) {
    for (int i = 0; i < MAX_FILES; i++) {
        if (files[i].in_use && strcmp_(files[i].name, filename) == 0) {
            files[i].in_use = 0;
            uart_puts("File deleted: ");
            uart_puts(filename);
            uart_putc('\n');
            return 0;
        }
    }
    uart_puts("Error: File not found.\n");
    return -1;
}

// 写入文件
int fs_write(const char *filename, const char *data, int size) {
    for (int i = 0; i < MAX_FILES; i++) {
        if (files[i].in_use && strcmp_(files[i].name, filename) == 0) {
            if (size > MAX_FILE_SIZE) {
                uart_puts("Error: File size exceeds maximum limit.\n");
                return -1;
            }
            strncpy_(files[i].data, data, size);
            files[i].size = size;
            uart_puts("Data written to file: ");
            uart_puts(filename);
            uart_putc('\n');
            return 0;
        }
    }
    uart_puts("Error: File not found.\n");
    return -1;
}

// 读取文件
int fs_read(const char *filename, char *buffer, int buffer_size) {
    for (int i = 0; i < MAX_FILES; i++) {
        if (files[i].in_use && strcmp_(files[i].name, filename) == 0) {
            int read_size = (files[i].size < buffer_size) ? files[i].size : buffer_size;
            strncpy_(buffer, files[i].data, read_size);
            buffer[read_size] = '\0';
            uart_puts("Data read from file: ");
            uart_puts(filename);
            uart_putc('\n');
            return read_size;
        }
    }
    uart_puts("Error: File not found.\n");
    return -1;
}

// 列出所有文件
void fs_list() {
    uart_puts("Files in file system:\n");
    for (int i = 0; i < MAX_FILES; i++) {
        if (files[i].in_use) {
            uart_puts("  ");
            uart_puts(files[i].name);
            uart_puts(" (Size: ");
            uart_putc('0' + files[i].size);
            uart_puts(" bytes)\n");
        }
    }
}
