// mutex.c

#include "kernel.h"

typedef struct {
    int locked;
} Mutex;

// 初始化互斥锁
void mutex_init(Mutex *m) {
    m->locked = 0;
}

// 获取互斥锁
void mutex_lock(Mutex *m) {
    while (__sync_lock_test_and_set(&m->locked, 1)) {
        // 自旋等待锁释放
    }
}

// 释放互斥锁
void mutex_unlock(Mutex *m) {
    __sync_lock_release(&m->locked);
}

// 演示互斥锁的使用
void mutex_demo() {
    static Mutex m;
    mutex_init(&m);

    uart_puts("Starting mutex demonstration...\n");

    uart_puts("Locking mutex...\n");
    mutex_lock(&m);
    uart_puts("Mutex locked! Critical section entered.\n");
    delay();
    uart_puts("Unlocking mutex...\n");
    mutex_unlock(&m);
    uart_puts("Mutex unlocked! Critical section exited.\n");
}
