// sync.c

#include "kernel.h"

typedef struct {
    int value;
} Semaphore;

// 初始化信号量
void semaphore_init(Semaphore *s, int init_value) {
    s->value = init_value;
}

// 等待信号量（P 操作）
void semaphore_wait(Semaphore *s) {
    while (s->value <= 0);  // 等待直到信号量值大于 0
    __sync_fetch_and_sub(&s->value, 1);
}

// 释放信号量（V 操作）
void semaphore_signal(Semaphore *s) {
    __sync_fetch_and_add(&s->value, 1);
}

// 演示信号量的使用
void sync_demo() {
    static Semaphore s;
    semaphore_init(&s, 1);

    uart_puts("Starting semaphore demonstration...\n");

    uart_puts("Waiting on semaphore...\n");
    semaphore_wait(&s);
    uart_puts("Semaphore acquired! Critical section entered.\n");
    delay();
    uart_puts("Signaling semaphore...\n");
    semaphore_signal(&s);
    uart_puts("Semaphore released! Critical section exited.\n");
}
